package net.pricefx.formulaengine;

import java.io.Closeable;
import java.util.Iterator;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;


public interface AbstractProducer<R> extends Callable<R> {

	/* timeout design:
		- the producer runs the hibernate search (SQL SELECT...) runs in its own thread (the callable)
	
		- the consumer is typically a formula element (groovy script); a FE has a default and user definable timeout.

		- the poll timeout is the max time the producer will wait for the consumer to pick up the offered result; the producer (callable) exits if this timeout is reached.
		  ==> if the consumer died, is no longer interested (polling), or is just too slow, then the producer exists;  this should prevent leaks.

		- reversely, if the producer takes longer than the poll timeout to make available the next result, then the polling is interrupted (see iterator.next);
	 	  this prevents consumer thread blocking, but it also means the consumer needs to check the result:
	 	  a) when result == null then there was a timeout if iterator.hasNext() returns true 
		  b) when result == null then all results have been already been delivered if iterator.hasNext() returns false
		  
		- if the FE times out before the search completes (in the DB), then the producer will stay alive until it finishes (difficult to interrupt the running DB query).
		  but at the first attempt to offer a result (first record in the search result set), it will time out and abort.
	*/
	
	public class ResultIterator implements Iterator<Object>, Closeable {
		private Iterator it;

		public ResultIterator(Iterator it) {
			this.it = it;
		}

		@Override
		public boolean hasNext() {
			return it.hasNext();
		}

		@Override
		public Object next() {
			return it.next();
		}

		@Override
		public void close() {
		}

	}
	
	public Future<R> getFuture();
	
	public ResultIterator getIterator();

}