package net.pricefx.formulaengine.scripting.inputbuilder;

import com.fasterxml.jackson.core.JsonProcessingException;
import net.pricefx.common.api.InputType;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.common.apibuilder.clicoll.CalculableLineItemCollectionBuilder;
import net.pricefx.lmformulaengine.Util;
import net.pricefx.server.dto.calculation.ConfiguratorEntry;
import net.pricefx.server.dto.calculation.ContextParameter;

import java.util.List;
import java.util.Map;

public abstract class AbstractInputBuilder<T extends AbstractInputBuilder, VT> {
    protected ContextParameter cp;
    protected final InputBuilderContext builderContext;

    public AbstractInputBuilder(InputBuilderContext builderContext, InputType type, String paramName) {
        this.builderContext = builderContext;
        cp = new ContextParameter(type, paramName,builderContext.getEngineContext().getTargetDate(), null);
    }

    public AbstractInputBuilder(InputBuilderContext builderContext) {
        this.builderContext = builderContext;
    }

    protected Object _getInput() {
        return builderContext.getEngineContext().getParameter(cp.getName());
    }

    /**
     * Builds the input field (<code>ContextParameter</code>).<p>
     * Use this on line items logics only.
     * Cannot be used on header logics.
     * <p>
     * Example:
     * <blockquote><pre>
     *     if (api.isSyntaxCheck()) {
     *     api.inputBuilderFactory()
     *             .createMultiTierEntryInputBuilder("MultiTier")
     *             .setLabel("MultiTier")
     *             .setValue([
     *                 "10" : "20",
     *                 "20" : "40"
     *             ])
     *             .getInput()
     * } else {
     *     api.input("MultiTier")
     * }
     * </pre></blockquote>
     *
     * @return The input field ({@link net.pricefx.server.dto.calculation.ContextParameter ContextParameter})
     * @since 5.1 - Collins
     */
    public final Object getInput() {
        if(builderContext.getEngineContext().getParameter(cp.getName()) == null)
            builderContext.getEngineContext().addMissingParameter(cp);  //this will effectively prevent the addMissingParameter call in the original functions; hence make sure our amended CP is added

        return _getInput();
    }

    /**
     * Sets the specified text as a label of the input field.<p>
     *
     * @param label The text that is displayed as a label of the input field.
     * @since 5.1 - Collins
     */
    public T setLabel(String label) {
        cp.setLabel(label);
        return (T) this;
    }

    /**
     * Sets the translation of the label.<p>
     * If the label translation is specified for a specific language then this translation is used when the UI is switched into that language.<p>
     * If the 2-letter language code string is left empty, it will be used as default translation in case the UI is switched to a language, for which you did not supply the corresponding translation.
     * <p>
     * Example:
     * <blockquote><pre>
     * def ce = api.createConfiguratorEntry()
     *
     * api.inputBuilderFactory()
     *      .createStringUserEntry("InputFieldName")
     *      .setLabel("English label")
     *      .setLabelTranslations(
     *      [
     *      "de": "Deutsche Bezeichnung",
     *      "es": "Etiqueta española"
     *      ]
     *      )
     *      .addToConfiguratorEntry(ce)
     * return ce
     * </pre></blockquote>
     *
     *
     * @param translations The map that defines language codes (2-letter <code>ISO 639-1</code>) and input field labels.
     * @see <a href="https://pricefx.atlassian.net/wiki/display/UNITY/Localization">Supported languages</a>, <a href="https://pricefx.atlassian.net/wiki/spaces/KB/pages/268763192/How+to+Set+Language+and+Locale+via+URL">How to Set Locale via URL</a>
     * @since 5.2 - Collins
     */
    public T setLabelTranslations(Map<String,String> translations) throws JsonProcessingException {
        cp.setLabelTranslationsFromMap(translations);
        return (T) this;
    }

    /**
     * Sets the input field as mandatory.
     * <p>The field is marked with an asterisk (*) in the user interface.
     * The user must enter a value to this input field otherwise the form cannot be saved.
     * <p>The default value: <code>false</code>
     *
     * @param required Set to <code>true</code> to mark the input field as required in the user interface.
     * @since 6.2 - Vesper
     */
    public T setRequired(Boolean required) {
        cp.setRequired(required);
        return (T) this;
    }

    /**
     * Sets the input field as read-only.<p>
     * The user cannot enter or edit the value in this field.
     * <p>The default value: <code>false</code>
     *
     * @param ro Set to <code>true</code> to set the input field as read-only.
     * @since 6.2 - Vesper
     */
    public T setReadOnly(Boolean ro) {
        cp.setReadOnly(ro);
        return (T) this;
    }

    /**
     * Sets the input field as always editable.<p>
     * Enables the user to update the input field after the Quote, Contract, or Rebate Agreement have been saved.
     * <p>Note: If the user has the DataIntegration user role assigned, this method may not work correctly.
     * Always editable input fields are supported in Unity UI only (not supported in Classic UI).
     * <p>The default value: <code>false</code>
     *
     * @param ae Set to <code>true</code> to set the input field as always editable.
     * @since 6.2 - Vesper
     */
    public T setAlwaysEditable(Boolean ae) {
        cp.setAlwaysEditable(ae);
        return (T) this;
    }

    /**
     * Prefills the input field with the specified value.<p>
     *
     * @param value The value to be prefilled in the input field.
     * @since 5.1 - Collins
     */
    public T setValue(VT value) {
        cp.setValue(value);
        return (T) this;
    }

    /**
     * Adds the specified hint next to the input field.
     * <p>Note: This method is available only for <code>MultiTierEntry</code>
     *
     * @param valueHint The text content of the hint.
     */
    public T setValueHint(String valueHint) {
        cp.setValueHint(valueHint);
        return (T) this;
    }

    /**
     * Prevents the form refresh when the value of the field is changed.
     *
     * <p>The default value: <code>false</code> - the form is refreshed every time when the user changes the input value.
     *
     * @param noRefresh Set to <code>true</code> to prevent the form refresh.
     * @since 5.1 - Collins
     */
    public T setNoRefresh(boolean noRefresh) {
        cp.setConfigParameter("noRefresh", noRefresh);
        return (T) this;
    }

    /**
     * Adds values to the input field (parameter).<p>
     * This can be used for example, for setting values of the drop-down input field.
     *
     * @param  configValues The list of values you want to add to the input field (parameter).
     */
    public T setConfigValues(List<String> configValues) throws JsonProcessingException {
        cp.addParameterConfigEntry(PriceFxInterface.FIELD_CONTEXTPARAM_CONFIG_VALUES, Util.getObjectMapper().writeValueAsString(configValues));
        return (T) this;
    }

    /**
     * Creates collapsible sections to organize input fields in the UI.
     * <p>You can assign <code>parameterGroup</code> to each parameter (input field) to categorize them into sections.
     * The order of parameter groups (sections) is based on the order in which they are "created" (declared) when the logic is executed.
     * There is one default group for all those that do not have <code>parameterGroup</code> assigned.
     *
     * @param pg  The name of the <code>parameterGroup</code>
     * @since 6.2 - Vesper
     */
    public T setParameterGroup(String pg) {
        cp.setParameterGroup(pg);
        return (T) this;
    }

    /**
     * Places the specified text to the input field as a placeholder.<p>
     * The placeholder text disappears once the user starts typing to this input field.
     * This gives a user the hint of the expected value.
     *
     * @param text  The text of the placeholder.
     * @since 7.1 - Bijou
     */
    public T setPlaceholderText(String text) {
        cp.setFormattingOption("placeholder",text);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     * <p>The default value: <code>false</code>
     *
     * @param importable
     */
    public T setImportable(boolean importable) {
        cp.setConfigParameter("importable",importable);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.
     * <p>The default value: <code>false</code>
     *
     * @param exportable
     */
    public T setExportable(boolean exportable) {
        cp.setConfigParameter("exportable",exportable);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param theme
     */
    public T setTheme(String theme) {
        cp.setFormattingOption("theme",theme);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param appearance
     */
    public T setAppearance(String appearance) {
        cp.setFormattingOption("appearance",appearance);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param size
     */
    public T setSize(String size) {
        cp.setFormattingOption("size",size);
        return (T) this;
    }

    /**
     * Displays the specified text when a user hovers mouse over the field label.<p>
     * Example:
     * <blockquote><pre>
     * def ce = api.createConfiguratorEntry()
     * api.inputBuilderFactory()
     * 			.createStringUserEntry("InputFieldName")
     * 			.setLabel("The Label")
     * 			.setTitle("The Title")
     * 			.addToConfiguratorEntry(ce)
     * return ce
     * </pre></blockquote>
     *
     * @param title The text of the title.
     */
    public T setTitle(String title) {
        cp.setFormattingOption("title",title);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param caption
     */
    public T setCaption(String caption) {
        cp.setFormattingOption("caption",caption);
        return (T) this;
    }

    /**
     * Displays a question mark icon next to the input field label and displays the specified <code>helpText</code> when a user hovers the mouse over this icon.<p>
     *
     * @param helpText  The text to be displayed when a user hovers over the icon.
     * @since 7.1 - Bijou
     */
    public T setHelpText(String helpText) {
        cp.setFormattingOption("helpText",helpText);
        return (T) this;
    }

    /**
     * Displays a question mark icon next to the field label and displays the "Learn more" link when a user hovers the mouse over the icon.<p>
     * Clicking on the "Learn more" text will open the specified <code>helpLink</code> destination.
     * We recommend using this function together with {@link #setHelpText(String)}
     *
     * @param helpLink  The target URL.
     * @since 7.1 - Bijou
     */
    public T setHelpLink(String helpLink) {
        cp.setFormattingOption("helpLink",helpLink);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param icon
     */
    public T setIcon(String icon) {
        cp.setFormattingOption("icon",icon);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param iconPosition
     */
    public T setIconPosition(String iconPosition) {
        cp.setFormattingOption("iconPosition",iconPosition);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param labelPlacement
     */
    public T setLabelPlacement(String labelPlacement) {
        cp.setFormattingOption("labelPlacement",labelPlacement);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param layoutGridName
     */
    public T setLayoutGridName(String layoutGridName) {
        cp.setFormattingOption("layoutGridName",layoutGridName);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param msg
     * @return
     */
    public T setErrorMessage(String msg) {
        cp.setConfigParameter("errorMessage",msg);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param msg
     * @return
     */
    public T setWarningMessage(String msg) {
        cp.setConfigParameter("warningMessage",msg);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param msg
     * @return
     */
    public T setSuccessMessage(String msg) {
        cp.setConfigParameter("successMessage",msg);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param ugv
     * @return
     */
    public T setUserGroupView(String ugv) {
        cp.setConfigParameter("userGroupView",ugv);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param uge
     * @return
     */
    public T setUserGroupEdit(String uge) {
        cp.setConfigParameter("userGroupEdit",uge);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param displayMode
     * @return
     */
    public T setDisplayMode(String displayMode) {
        cp.setFormattingOption("displayMode",displayMode);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param accessCode
     * @return
     */
    public T setAccessCode(String accessCode) {
        cp.setFormattingOption("accessCode",accessCode);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param align
     * @return
     */
    public T setAlign(String align) {
        cp.setFormattingOption("align",align);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param labelStyle
     * @return
     */
    public T setLabelStyle(String labelStyle) {
        cp.setFormattingOption("labelStyle",labelStyle);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param autoComplete
     * @return
     */
    public T setAutoComplete(boolean autoComplete) {
        cp.setConfigParameter("autoComplete",autoComplete);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param autoFocus
     * @return
     */
    public T setAutoFocus(boolean autoFocus) {
        cp.setConfigParameter("autoFocus",autoFocus);
        return (T) this;
    }

    /**
     * N/A - the method has not been implemented in the user interface yet.<p>
     *
     * @param disabled
     * @return
     */
    public T setDisabled(boolean disabled) {
        cp.setConfigParameter("disabled",disabled);
        return (T) this;
    }

    /**
     * Creates the custom name data attribute in the form's HTML.<p>
     * The resulting HTML: <code>data-custom-AttributeName="AttributeValue"</code>.
     *
     * @param name  The name of the custom attribute.
     * @since 7.1 - Bijou
     */
    public T setCustomAttributeName(String name) {
        cp.setFormattingOption("customAttributeName",name);
        return (T) this;
    }

    /**
     * Creates the custom value data attribute in the form's HTML.<p>
     * For example, <code>data-custom-AttributeName="AttributeValue"</code>.
     *
     * @param value  The value of the custom attribute.
     * @since 7.1 - Bijou
     */
    public T setCustomAttributeValue(String value) {
        cp.setFormattingOption("customAttributeValue",value);
        return (T) this;
    }

    /**
     * How much space should the element take out of a row when a InputType 'ROW' is used.
     * @param flex value to define how the element will grow or shrink to fit the space available.
     */
    public T setFlex(int flex) {
        cp.setFormattingOption("flex", flex);
        return (T) this;
    }

    /**
     * Builds the input field map (<code>ContextParameter</code> converted to a map) that can be used by the <code>{@link #addOrUpdateInput(CalculableLineItemCollectionBuilder, String)}</code> method in a header logic.
     *
     * <p>Example:
     * <blockquote><pre>
     *     def deliveryType = api.inputBuilderFactory()
     *         .createOptionEntry("DeliveryType")
     *         .setLabel("Delivery Type")
     *         .setRequired(true)
     *         .setOptions(["Express", "Standard"])
     *         .setLabels(["Express": "Express Delivery", "Standard": "Standard Delivery"])
     *         .buildMap()
     *
     * quoteProcessor.addOrUpdateInput(deliveryType)
     * </pre></blockquote>
     *
     * @return The input field map.
     * @since 5.1 - Collins
     */
    public Map<String,Object> buildMap() {
        return Util.getObjectMapper().convertValue(cp, Map.class);
    }

    /**
     * Builds the input field ({@link net.pricefx.server.dto.calculation.ContextParameter ContextParameter}).
     * <p>
     * Example:
     * <blockquote><pre>
     *
     def ce = api.createConfiguratorEntry()

     def meatParameter = api.inputBuilderFactory()
     .createOptionEntry("Meat")
     .setOptions(["Beef", "Lamb", "Pork"])
     .buildContextParameter()

     return ce.createParameter(meatParameter,{"Beef"})
     * </pre></blockquote>
     *
     * @return The input field ({@link net.pricefx.server.dto.calculation.ContextParameter ContextParameter}).
     * @since 5.1 - Collins
     */
    public ContextParameter buildContextParameter() {
        return cp;
    }

    /**
     *  Adds or updates the input field in the specified header line item.<p>
     *  Available for: Quotes, RebateAgreements and Contracts.
     *
     *  <p>Example - adds the input to the QuoteBuilder at the root level:
     *  <p><code>.addOrUpdateInput(quoteProcessor, "ROOT")</code>
     *  <p>
     *
     * @param clicBuilder  The clicBuilder object (quoteProcessor, raProcessor, cProcessor).
     * @param lineId  The <code>lineId</code>.
     * @since 5.1 - Collins
     */
    public T addOrUpdateInput(CalculableLineItemCollectionBuilder clicBuilder, String lineId) {
        if(clicBuilder != null)
            clicBuilder.addOrUpdateInput(lineId,buildMap());
        return (T) this;
    }

    /**
     * Adds the created object (the input field) to the specified configurator entry (section).
     * <p>Example:
     * <blockquote><pre>
     * def ce = api.createConfiguratorEntry()
     * api.inputBuilderFactory()
     *     .createStringUserEntry("StringUserEntry")
     *     .setLabel("String User Entry")
     *     .addToConfiguratorEntry(ce)
     * return ce
     *</pre></blockquote>
     *
     * @param ce  The <code>ConfiguratorEntry</code> where you want to add the created input field object.
     * @since 5.1 - Collins
     */
    public T addToConfiguratorEntry(ConfiguratorEntry ce) {
        ce.addParameter(cp);
        return (T) this;
    }
}
